"""Zellulärer Automat für ein einfaches Diffusionsmodell."""

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.animation

# Erzeuge ein quadratisches NxN-Spielfeld,
# Der Wert jedes Feldes gibt die Konzentration einer bestimmten Substanz an.
N = 500
konzentration = np.empty((N, N))

# Der Parameter gibt die Stärke der Diffusion an.
D = 0.22

# Definiere die Indizes der Nachbarschaftsbeziehung
nachbarschaft = [(-1, 0), (+1, 0), (0, -1), (0, +1)]

# Erstelle ein Plot-Fenster und erzeuge die Darstellung des Spielfelds.
fig = plt.figure()
ax = fig.add_subplot(1, 1, 1)
colormap = matplotlib.colors.LinearSegmentedColormap.from_list(\
                              'mycolormap', ["lightgray", "red"], 255)
image = ax.imshow(konzentration, clim=(0, 1), cmap=colormap)
fig.tight_layout()


def init():
    """Initialisiert die Grafikausgabe beipsielsweise nach Größenänderung des
    Ausgabefensters."""
    return image,


def update(frame):
    """Berechnet den nächsten Zeitschritt und gibt ein Tupel der zu
    aktualisierenden Grafikelemente zurück."""
    global konzentration, t

    # Im nullten Zeitschritt soll zunaechst nur der Anfangszustand
    # dargestellt werden.
    if t == 0:
        image.set_array(konzentration)
        t += 1
        return image,

    # Zwischen benachbarten Zellen findet ein Konzentrationsausgleich statt,
    # der proportional zur Differenz der Konzentration ist.
    nachbarn = sum(np.roll(konzentration, (i, j), axis=(0, 1))
                   for (i, j) in nachbarschaft)
    konzentration += D * (nachbarn - konzentration * len(nachbarschaft))

    # Akualisiere das Bild
    image.set_array(konzentration)

    # Erhöhe den Zähler für die Zeitschritte
    t += 1

    return image,


# Zeitzähler wird auf Null gesetzt.
t = 0

# Im Anfangszustand ist die Konzentration Null. In der Mitte des Feldes
# ist in einem quadratischen Bereich die Konzentration Eins.
konzentration[:] = 0
konzentration[220:280, 220:280] = 1

# Erzeuge die Animation und starte sie.
ani = matplotlib.animation.FuncAnimation(fig, update, init_func=init,
                                         interval=1, blit=True)
plt.show()
